package demo.rest;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;

import javax.ws.rs.core.MediaType;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.commons.httpclient.methods.multipart.FilePart;
import org.apache.commons.httpclient.methods.multipart.MultipartRequestEntity;
import org.apache.commons.httpclient.methods.multipart.Part;
import org.apache.commons.httpclient.methods.multipart.StringPart;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;

public class BulkUploadClient {
  
  public static void main(String[] args)
  {  
    try
    {    
      // Modify the login() method to specify user name and password.
      HttpClient httpClient = login();

      String url = "http://{domain}/networking/rest/bulk/Customer";      
      String importXml =
        "<platform>" +
           "<bulk>" +
              "<mapping>CustomerMap</mapping>" +                               
              // For Prospects, add: "<leadSource>...</leadSource>" +
              // For Activities, add: "<ownerName>...</ownerName>" +
           "</bulk>" +
        "</platform>" ;
      
      // XML part
      StringPart sp = new StringPart("bulk", importXml);
      sp.setContentType(MediaType.APPLICATION_XML);
       
      // File part
      File file = new File("c:/document/customers.csv");     
      
      // Multipart wrapper
      Part[] part = {                        
                      new FilePart(file.getName(),file),
                      sp
                    };
                
      // Create the request          
      PostMethod postMethod = new PostMethod(url);
      MultipartRequestEntity requestEntity =
          new MultipartRequestEntity( part, postMethod.getParams() );
      postMethod.setRequestEntity(requestEntity);         

      // Send it
      int status  = httpClient.executeMethod(postMethod);   
      System.out.println("HTTP Status = " + status);            
     
      String responseXml = postMethod.getResponseBodyAsString();
      String jobId = getValue("platform/message/id", responseXml);

      System.out.println("Use the following URL to check job status:");
      System.out.println("https://{domain}/networking/rest/status/import/"
                       + jobId);          
    }
      catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  /**
   * Log into the platform. After doing so, session ID is automatically
   * set for all future requests that use the HttpClient. (The server sets up
   * a cookie string for the session, and returns it in the response. The 
   * HttpClient instance saves that string and adds it to subsequent requests.
   * Reusing an instance of HttpClient, in effect, defines a "session".)
   *
   * @return The HttpClient object when the login succeeds.
   * @throws An exception if the login fails.
   */ 
 public static HttpClient login() throws Exception
  {
      String url = "http://qamain.longjump.com/networking/rest/login";
      PostMethod postMethod = new PostMethod(url);      
      String loginXml =
            "<platform>" +
               "<login>" +
                  "<userName>yourLoginID</userName>"  + 
                  "<password>YourPassword</password>" +
               "</login>" +
            "</platform>" ;

      StringRequestEntity reqXml =
        new StringRequestEntity(loginXml, MediaType.APPLICATION_XML, "UTF-8");
      postMethod.setRequestEntity(reqXml);
      
      // Log in
      HttpClient httpClient = new HttpClient();      
      int status = httpClient.executeMethod(postMethod);
      if (status != 200) {              
         throw new Exception("Login failed with HTTP status " + status);
      }
      
      return httpClient;
  }

  
  /**
   * Given an XPath expression and XML, parse for a specific value.
   * @param xpath_expr  An XPATH expression. 
   *                    For example: "/platform/message/description"
   * @param xml The XML string to be parsed.
   */
  public static String getValue(String xpath_expr, String xml) 
  {
    try {
      DocumentBuilderFactory docFactory =
          DocumentBuilderFactory.newInstance();
      docFactory.setNamespaceAware(true);
      DocumentBuilder docbuilder = docFactory.newDocumentBuilder();
      InputStream is = new ByteArrayInputStream(xml.getBytes());

      Document doc = docbuilder.parse(new InputSource(is));
      XPathFactory factory = XPathFactory.newInstance();
      XPath xpath = factory.newXPath();
      XPathExpression expr = xpath.compile(xpath_expr);
      Object result = expr.evaluate(doc, XPathConstants.STRING);
      return result.toString();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
    return null;
  }
  
}
